#include "stars.h"
#include <ctime>
#include <cmath>
#include <cstdlib>

float random_float(){
  return ((float)rand() / RAND_MAX);
}


void Stars::Initialize(GLuint n){
    n_ = n;
    time_ = 0.0f;
    srand(time(NULL));
    glGenVertexArrays(1, &vao_);
    glBindVertexArray(vao_);

    glGenBuffers(1, &vertex_buffer_);
    glBindBuffer(GL_ARRAY_BUFFER, vertex_buffer_);
    glBufferData(GL_ARRAY_BUFFER, n_ * sizeof(ColorVertex), NULL, GL_STATIC_DRAW);

    ColorVertex * star = (ColorVertex*) glMapBufferRange(GL_ARRAY_BUFFER, 0,
                                                         n_ * sizeof(ColorVertex),
                                                         GL_MAP_WRITE_BIT | GL_MAP_INVALIDATE_BUFFER_BIT);
    for (unsigned int i = 0; i < n_; i++){
    star[i].position[0] = (random_float() * 2.0f - 1.0f) * 100.0f;
    star[i].position[1] = (random_float() * 2.0f - 1.0f) * 100.0f;
    star[i].position[2] = random_float();
    star[i].position[3] = 1.0f;
    star[i].color[0] = 0.8f + random_float() * 0.2f;
    star[i].color[1] = 0.8f + random_float() * 0.2f;
    star[i].color[2] = 0.8f + random_float() * 0.2f;
    star[i].color[3] = 1.0f;
    }

    glUnmapBuffer(GL_ARRAY_BUFFER);

    glVertexAttribPointer(0, 4, GL_FLOAT, GL_FALSE, sizeof(ColorVertex), NULL);
    glVertexAttribPointer(1, 4, GL_FLOAT, GL_FALSE, sizeof(ColorVertex), (void *) sizeof(star[0].position));
    glEnableVertexAttribArray(0);
    glEnableVertexAttribArray(1);
}

void Stars::Draw(const TimeTextureProjectionProgram &pr){
    glUseProgram(pr);

    pr.SetTime(time_);

    glBindVertexArray(vao_);

    glActiveTexture(texture_unit_);
    glBindTexture(GL_TEXTURE_2D, texture_);

    glEnable(GL_BLEND);
    glBlendFunc(GL_ONE, GL_ONE);

    glEnable(GL_PROGRAM_POINT_SIZE);

    glDrawArrays(GL_POINTS, 0, n_);

    glDisable(GL_BLEND);
    glDisable(GL_PROGRAM_POINT_SIZE);
    glBindVertexArray(0);
    glUseProgram(0);

}

void Stars::Move(GLfloat t){
    time_ = t;
}
