#include "cube.h"
#include "vertices.h"

void Cube::Initialize(){

    const MaterialVertex kVertices[24] =
    {
      //front
      {{-0.75f, 0.75f, 0.75f, 1.0f}, { 0.0f, 0.0f, 1.0f}},
      {{ 0.75f, 0.75f, 0.75f, 1.0f}, { 0.0f, 0.0f, 1.0f}},
      {{ 0.75f,-0.75f, 0.75f, 1.0f}, { 0.0f, 0.0f, 1.0f}},
      {{-0.75f,-0.75f, 0.75f, 1.0f}, { 0.0f, 0.0f, 1.0f}},
      // back
      {{ 0.75f, 0.75f,-0.75f, 1.0f}, { 0.0f, 0.0f,-1.0f}},
      {{-0.75f, 0.75f,-0.75f, 1.0f}, { 0.0f, 0.0f,-1.0f}},
      {{-0.75f,-0.75f,-0.75f, 1.0f}, { 0.0f, 0.0f,-1.0f}},
      {{ 0.75f,-0.75f,-0.75f, 1.0f}, { 0.0f, 0.0f,-1.0f}},
      // top
      {{-0.75f, 0.75f,-0.75f, 1.0f}, { 0.0f, 1.0f, 0.0f}},
      {{ 0.75f, 0.75f,-0.75f, 1.0f}, { 0.0f, 1.0f, 0.0f}},
      {{ 0.75f, 0.75f, 0.75f, 1.0f}, { 0.0f, 1.0f, 0.0f}},
      {{-0.75f, 0.75f, 0.75f, 1.0f}, { 0.0f, 1.0f, 0.0f}},
      // bottom
      {{ 0.75f,-0.75f,-0.75f, 1.0f}, { 0.0f,-1.0f, 0.0f}},
      {{-0.75f,-0.75f,-0.75f, 1.0f}, { 0.0f,-1.0f, 0.0f}},
      {{-0.75f,-0.75f, 0.75f, 1.0f}, { 0.0f,-1.0f, 0.0f}},
      {{ 0.75f,-0.75f, 0.75f, 1.0f}, { 0.0f,-1.0f, 0.0f}},
      // left
      {{-0.75f, 0.75f,-0.75f, 1.0f}, {-1.0f, 0.0f, 0.0f}},
      {{-0.75f, 0.75f, 0.75f, 1.0f}, {-1.0f, 0.0f, 0.0f}},
      {{-0.75f,-0.75f, 0.75f, 1.0f}, {-1.0f, 0.0f, 0.0f}},
      {{-0.75f,-0.75f,-0.75f, 1.0f}, {-1.0f, 0.0f, 0.0f}},
      // right
      {{ 0.75f, 0.75f, 0.75f, 1.0f}, { 1.0f, 0.0f, 0.0f}},
      {{ 0.75f, 0.75f,-0.75f, 1.0f}, { 1.0f, 0.0f, 0.0f}},
      {{ 0.75f,-0.75f,-0.75f, 1.0f}, { 1.0f, 0.0f, 0.0f}},
      {{ 0.75f,-0.75f, 0.75f, 1.0f}, { 1.0f, 0.0f, 0.0f}}
    };

    const GLuint kIndices[36] =
    {
      0, 3, 1,  1, 3, 2, // front
      4, 7, 5,  5, 7, 6, // back
      8,11, 9,  9,11,10, // top
      12,15,13, 13,15,14, // bottom
      16,19,17, 17,19,18, // left
      20,23,21, 21,23,22  // right
    };

    glGenVertexArrays(1, &vao_);
    glBindVertexArray(vao_);

    glGenBuffers(1, &vertex_buffer_);
    glBindBuffer(GL_ARRAY_BUFFER, vertex_buffer_);
    glBufferData(GL_ARRAY_BUFFER, sizeof(kVertices), kVertices, GL_STATIC_DRAW);
    glVertexAttribPointer(0, 4, GL_FLOAT, GL_FALSE, sizeof(kVertices[0]), nullptr);
    glEnableVertexAttribArray(0);

    glGenBuffers(1, &index_buffer_);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, index_buffer_);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(kIndices), kIndices, GL_STATIC_DRAW);
    glVertexAttribPointer(3, 3, GL_FLOAT, GL_FALSE, sizeof(kVertices[0]),  reinterpret_cast<GLvoid*>( sizeof(kVertices[0].position) ) );
    glEnableVertexAttribArray(3);
     glBindVertexArray(0);
}


void Cube::Move(float delta_t){
    if (!animated_) return;
    angle_ += delta_t * velocity_;
    if(angle_>360) angle_ -= 360;
    if(angle_<-360) angle_ += 360;

    model_matrix_.SetUnitMatrix();
    model_matrix_.Scale(0.5, 0.5, 0.5);
    model_matrix_.RotateAboutX(2*angle_);
    model_matrix_.Translate(-3, 0, 2);
    model_matrix_.RotateAboutY(angle_);
    normal_matrix_.SetUnitMatrix();
    normal_matrix_.RotateAboutY(-angle_);
    normal_matrix_.RotateAboutX(-2*angle_);
    normal_matrix_.Scale(2.0, 2.0, 2.0);
}


void Cube::Draw(const PointLightTransparentProgram &program)const{
    glBindVertexArray(vao_);

    glUseProgram(program);

    program.SetModelMatrix(model_matrix_);
    program.SetNormalMatrix(normal_matrix_);
    program.SetMaterial(material_);

    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);
    glFrontFace(GL_CCW);

    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glDepthMask(0);

    glDrawElements(GL_TRIANGLES, 36, GL_UNSIGNED_INT, nullptr);

    glDisable(GL_CULL_FACE);
    glDisable(GL_BLEND);
    glDepthMask(1);

     glBindVertexArray(0);

    glUseProgram(0);

}
