#include <cmath>

#include "torus.h"


void Torus::Move(GLfloat delta_t){
    if (!animated_) return;
    angle_ += delta_t * velocity_;
    if(angle_>360) angle_ -= 360;
    if(angle_<-360) angle_ += 360;
    model_matrix_.SetUnitMatrix();
    model_matrix_.RotateAboutX(angle_);
    model_matrix_.RotateAboutY(angle_);
    normal_matrix_.SetUnitMatrix();
    normal_matrix_.RotateAboutY(-angle_);
    normal_matrix_.RotateAboutX(-angle_);
}

void Torus::Initialize(GLuint n, GLuint m, GLfloat R, GLfloat r){
    n_ =n ;
    m_ = m;
    NormalTextureVertex * vertices=new NormalTextureVertex[(m_ + 1)*(n_ + 1)];
    GLuint i, j;
    for (i=0;i<=n_;i++) {
      GLfloat phi = 2*static_cast<GLfloat>(M_PI)/static_cast<GLfloat>(n_)*i;
      for (j=0;j<=m_;j++){
        GLfloat theta = 2 * static_cast<GLfloat>(M_PI)/static_cast<GLfloat>(m_)*j;
        vertices[i*(m_ + 1)+j].position[0]=(R + r*cosf(phi))*sinf(theta);
        vertices[i*(m_ + 1)+j].position[1]=r*sinf(phi);
        vertices[i*(m_ + 1)+j].position[2]=(R + r*cosf(phi))*cosf(theta);
        vertices[i*(m_ + 1)+j].position[3]=1.0f;
        vertices[i*(m_ + 1)+j].texture[0]=static_cast<GLfloat>(j)/static_cast<GLfloat>(m_);
        vertices[i*(m_ + 1)+j].texture[1]=static_cast<GLfloat>(i)/static_cast<GLfloat>(n_);
        vertices[i*(m_+1)+j].normal[0]=cosf(phi)*sinf(theta);
        vertices[i*(m_+1)+j].normal[1]=sinf(phi);
        vertices[i*(m_+1)+j].normal[2]=cosf(phi)*cosf(theta);
      }
    }



    GLuint * indices=new GLuint[2*n_*(m_ + 1)];

    unsigned int k=0;

    for(i=0; i<=n_ - 1; i++){
      for(j=0; j<=m_; j++){
        indices[2*(i*(m_ + 1)+j)]=k;
        indices[2*(i*(m_ + 1)+j)+1]=k+m_+1;
        k++;
      }
    }

    glGenVertexArrays(1, &vao_);
    glBindVertexArray(vao_);


    glGenBuffers(1, &vertex_buffer_);
    glBindBuffer(GL_ARRAY_BUFFER, vertex_buffer_);
    glBufferData(GL_ARRAY_BUFFER,
            (m_+1)*(n_+1)*sizeof(vertices[0]),
            vertices,
            GL_STATIC_DRAW);
    glVertexAttribPointer(0, 4, GL_FLOAT, GL_FALSE, sizeof(vertices[0]), nullptr);
    glEnableVertexAttribArray(0);
    glVertexAttribPointer(2, 2, GL_FLOAT, GL_FALSE, sizeof(vertices[0]),
            reinterpret_cast<GLvoid*> (sizeof(vertices[0].position)) );
    glEnableVertexAttribArray(2);
    glVertexAttribPointer(3, 3, GL_FLOAT, GL_FALSE, sizeof(vertices[0]),
            reinterpret_cast<GLvoid*> (sizeof(vertices[0].position)+sizeof(vertices[0].texture)) );
    glEnableVertexAttribArray(3);

    glGenBuffers(1, &index_buffer_);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, index_buffer_);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER,
                 (m_+1)*n_*2*sizeof(GLuint),
                 indices,
                 GL_STATIC_DRAW
                 );
    delete [] vertices;
    delete [] indices;
    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glBindVertexArray(0);

}

void Torus::Draw(const LightProgram &program) const{
    glBindVertexArray(vao_);

    glUseProgram(program);
    program.SetModelMatrix(model_matrix_);
    program.SetNormalMatrix(normal_matrix_);
    program.SetMaterial(material_);


    glActiveTexture(texture_unit_);
    glBindTexture(GL_TEXTURE_2D, texture_);

    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);
    glFrontFace(GL_CW);


    for (GLuint i=0;i<n_;i++){
      glDrawElements(GL_TRIANGLE_STRIP,
                     static_cast<GLint>( 2*(m_ + 1)),
                     GL_UNSIGNED_INT,
                     reinterpret_cast<GLvoid*>(sizeof(GLuint) * 2 * i * (m_ + 1))
                     );
    }
    glDisable(GL_CULL_FACE);

    glBindVertexArray(0);
    glUseProgram(0);


}
