#include "transparentmaterialmodelprogram.h"

void TransparentMaterialModelProgram::Initialize(const char *vertex_shader_file, const char *fragment_shader_file){
    CameraProgram::Initialize(vertex_shader_file, fragment_shader_file);
    model_matrix_location_ = GetUniformLocationOrDie("model_matrix");
    material_locations_.emission = GetUniformLocationOrDie("material.emission");
    material_locations_.ambient = GetUniformLocationOrDie("material.ambient");
    material_locations_.diffuse = GetUniformLocationOrDie("material.diffuse");
    material_locations_.specular = GetUniformLocationOrDie("material.specular");
    material_locations_.shininess = GetUniformLocationOrDie("material.shininess");
}

void TransparentMaterialModelProgram::SetModelMatrix(const Mat4 & matrix) const{
    glUniformMatrix4fv(model_matrix_location_, 1, GL_FALSE, matrix);
}

void TransparentMaterialModelProgram::SetMaterial(const Material & material) const{
    glUniform4fv(material_locations_.ambient, 1, material.ambient);
    glUniform4fv(material_locations_.emission, 1, material.emission);
    glUniform4fv(material_locations_.diffuse, 1, material.diffuse);
    glUniform4fv(material_locations_.specular, 1, material.specular);
    glUniform1f(material_locations_.shininess, material.shininess);
}
